#!/usr/bin/env python

"""
A handler to help with testing.

Copyright (C) 2014, 2015 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from imiptools.client import ClientForObject
from imiptools.data import Object, get_address, parse_object
from imiptools.mail import Messenger
import imip_store
import sys

class TestClient(ClientForObject):

    """
    A content handler for use in testing, as opposed to operating within the
    mail processing pipeline.
    """

    # Action methods.

    def handle_request(self, accept):

        """
        Process the current request for the current user. Return whether any
        action was taken.
        """

        # Reply only on behalf of this user.

        attendee_attr = self.update_participation(self.obj, accept and "ACCEPTED" or "DECLINED")

        if not attendee_attr:
            return None

        # NOTE: This is a simpler form of the code in imipweb.client.

        organiser = get_address(self.obj.get_value("ORGANIZER"))

        self.obj["ATTENDEE"] = [(self.user, attendee_attr)]
        self.update_dtstamp()
        self.set_sequence(False)

        message = self.messenger.make_outgoing_message(
            [self.obj.to_part("REPLY")],
            [organiser],
            outgoing_bcc=get_address(self.user)
            )

        return message.as_string()

# A simple main program that attempts to handle a stored request, writing the
# response message to standard output.

if __name__ == "__main__":
    try:
        minargs = 3; maxargs = 5
        accept, store_dir, user, uid, recurrenceid = (sys.argv[1:maxargs+1] + ([None] * (maxargs - minargs)))[:maxargs]
    except ValueError:
        print >>sys.stderr, "Need 'accept' or 'decline', a store directory, user URI, event UID and optional RECURRENCE-ID."
        print >>sys.stderr, "The RECURRENCE-ID must be in the form employed by the store."
        print >>sys.stderr
        print >>sys.stderr, "Alternatively, omit the UID and RECURRENCE-ID and provide event-only details on standard input"
        print >>sys.stderr, "to force the script to handle an event not already present in the store."
        sys.exit(1)

    store = imip_store.FileStore(store_dir)

    if uid is not None:
        fragment = store.get_event(user, uid, recurrenceid)

        if not fragment:
            print >>sys.stderr, "No such event:", uid, recurrenceid
            sys.exit(1)
    else:
        fragment = parse_object(sys.stdin, "utf-8")

    obj = Object(fragment)
    handler = TestClient(obj, user, Messenger(), store)
    response = handler.handle_request(accept == "accept")

    if response:
        if uid is not None:
            store.dequeue_request(user, uid, recurrenceid)
        print response
    else:
        sys.exit(1)

# vim: tabstop=4 expandtab shiftwidth=4
